import 'dart:convert';
import 'dart:io';

import 'package:apus_drones_mobile/constants/UserType.dart';
import 'package:apus_drones_mobile/models/CreateProductModel.dart';
import 'package:apus_drones_mobile/models/PictureToUploadModel.dart';
import 'package:apus_drones_mobile/models/ProductModel.dart';
import 'package:apus_drones_mobile/models/UpdateProductModel.dart';
import 'package:apus_drones_mobile/screens/Appbar.dart';
import 'package:apus_drones_mobile/screens/PartnerProductListScreen.dart';
import 'package:apus_drones_mobile/services/ProductService.dart';
import 'package:apus_drones_mobile/utils/money.dart';
import 'package:apus_drones_mobile/widgets/BotNavBar.dart';
import 'package:apus_drones_mobile/widgets/PicturesGrid.dart';
import 'package:brasil_fields/brasil_fields.dart';
import 'package:collection/collection.dart';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:path/path.dart' as Path;

class EditProductScreen extends StatefulWidget {
  final int? id;
  const EditProductScreen({Key? key, this.id}) : super(key: key);

  @override
  _EditProductScreenState createState() => _EditProductScreenState();
}

class _EditProductScreenState extends State<EditProductScreen> {
  ProductService service = ProductService();
  ProductModel? currentProduct;
  List<File>? pictures;
  double price = 0;
  double weight = 0;
  int quantity = 0;
  List<String> removedImagesUrls = [];

  TextEditingController nameController = TextEditingController();
  TextEditingController descriptionController = TextEditingController();
  TextEditingController priceController = TextEditingController();
  TextEditingController weightController = TextEditingController();
  TextEditingController quantityController = TextEditingController();

  void _fetchProduct() {
    int? widgetId = widget.id;
    if (widgetId == null) {
      return;
    }

    service.getProduct(widgetId).then((data) => this.setState(() {
          currentProduct = data;
          nameController.value = TextEditingValue(text: data.name);
          descriptionController.value =
              TextEditingValue(text: data.description);
          priceController.value =
              TextEditingValue(text: convertNumToReais(data.price));
          weightController.value = TextEditingValue(
              text: data.weight.toString().replaceAll('.', ','));
          quantityController.value =
              TextEditingValue(text: data.quantity.toString());
        }));
  }

  void _save(String name, String description, double price, double weight,
      int quantity, List<File>? pictures) async {
    await (currentProduct == null
        ? _postProduct(name, description, price, weight, quantity)
        : _updateProduct(name, description, price, weight, quantity));

    Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) => PartnerProductListScreen(),
      ),
    );
  }

  Future<List<PictureToUploadModel>> _getPicturesModels() async {
    final _pictures = pictures;
    return _pictures != null
        ? (await Future.wait(
            _pictures.map(
              (p) async {
                List<int> picBytes = await p.readAsBytes();
                String base64 = base64Encode(picBytes);
                return PictureToUploadModel(
                  base64: base64,
                  fileName: Path.basename(p.path),
                  mainFile: false,
                );
              },
            ),
          ))
            .toList()
        : [];
  }

  Future<void> _postProduct(String name, String description, double price,
      double weight, int quantity) async {
    final _pictures = pictures;
    List<PictureToUploadModel> files = await _getPicturesModels();

    CreateProductModel product = new CreateProductModel(
      name: name,
      description: description,
      price: price,
      status: 'ACTIVE',
      weight: weight,
      createdAt: DateTime.now(),
      quantity: quantity,
      files: files,
      imagesUrls: [],
    );

    await service.postProduct(product);
  }

  Future<void> _updateProduct(String name, String description, double price,
      double weight, int quantity) async {
    ProductModel? _product = currentProduct;
    if (_product == null) {
      return;
    }

    List<PictureToUploadModel> files = await _getPicturesModels();

    UpdateProductModel newProduct = new UpdateProductModel(
      id: _product.id,
      name: name,
      description: description,
      price: price,
      status: 'ACTIVE',
      weight: weight,
      removedImagesUrls: removedImagesUrls,
      files: files,
      quantity: quantity,
    );

    await service.patchProduct(newProduct);
  }

  void _openFileExplorer() async {
    FilePickerResult? result = await FilePicker.platform.pickFiles(
      type: FileType.image,
      allowMultiple: true,
    );
    if (result != null) {
      List<File> files = result.paths.map((path) => File(path ?? '')).toList();
      setState(() {
        pictures = EqualitySet<File>.from(
                EqualityBy((File f) => f.path), [...pictures ?? [], ...files])
            .toList();
      });
    }
  }

  void _confirmRemovePicture(dynamic picture) {
    showDialog(
        context: context,
        builder: (BuildContext context) => AlertDialog(
                title: Text('Excluir foto'),
                content: Text('Confirma a exclusão da foto?'),
                actions: [
                  TextButton(
                      onPressed: () {
                        Navigator.of(context).pop();
                      },
                      child: Text('Cancelar')),
                  TextButton(
                      onPressed: () {
                        _removePicture(picture);
                        Navigator.of(context).pop();
                      },
                      child: Text('Sim'))
                ]));
  }

  void _removePicture(dynamic picture) {
    final _picture = picture;
    final _currentProduct = currentProduct;
    final _removedImagesUrls = removedImagesUrls;

    if (_picture is File) {
      setState(() {
        pictures = (pictures ?? [])
            .where((element) => element.path != picture.path)
            .toList();
      });
    } else if (_picture is String && _currentProduct != null) {
      (_currentProduct.imagesUrls ?? []).remove(_picture);
      _removedImagesUrls.add(_picture);

      setState(() {
        currentProduct = _currentProduct;
        removedImagesUrls = _removedImagesUrls;
      });
    }
  }

  @override
  void initState() {
    super.initState();
    this._fetchProduct();
    priceController.addListener(() {
      setState(() {
        price = convertReaisToNum(priceController.text).toDouble();
      });
    });
    weightController.addListener(() {
      setState(() {
        weight =
            double.tryParse(weightController.text.replaceAll(',', '.')) ?? 0;
      });
    });
    quantityController.addListener(() {
      setState(() {
        quantity = int.tryParse(quantityController.text) ?? 0;
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    final _product = currentProduct;
    return Scaffold(
      appBar: Appbar(title: '${_product == null ? 'Criar' : 'Editar'} produto'),
      body: SingleChildScrollView(
        child: Container(
          height: MediaQuery.of(context).size.height - 60,
          width: MediaQuery.of(context).size.width,
          padding: const EdgeInsets.only(top: 25, right: 25, left: 25),
          child: Column(
            children: [
              TextField(
                controller: nameController,
                decoration: InputDecoration(
                  labelText: 'Nome do Produto',
                  border: OutlineInputBorder(),
                ),
              ),
              SizedBox(
                height: 30,
              ),
              TextField(
                controller: descriptionController,
                decoration: InputDecoration(
                  labelText: 'Descrição',
                  border: OutlineInputBorder(),
                ),
              ),
              SizedBox(
                height: 30,
              ),
              Row(
                children: [
                  SizedBox(
                    width: 161,
                    child: TextField(
                      controller: priceController,
                      decoration: InputDecoration(
                        labelText: 'Valor (R\$)',
                        border: OutlineInputBorder(),
                      ),
                      keyboardType: TextInputType.number,
                      inputFormatters: [
                        FilteringTextInputFormatter.digitsOnly,
                        RealInputFormatter(centavos: true, moeda: true)
                      ],
                    ),
                  ),
                  SizedBox(width: 20),
                  SizedBox(
                    width: 80,
                    child: TextField(
                      controller: weightController,
                      decoration: InputDecoration(
                        labelText: 'Peso (g)',
                        border: OutlineInputBorder(),
                      ),
                      keyboardType: TextInputType.number,
                      inputFormatters: [
                        FilteringTextInputFormatter.digitsOnly,
                        LengthLimitingTextInputFormatter(4),
                        PesoInputFormatter()
                      ],
                    ),
                  ),
                  SizedBox(width: 20),
                  SizedBox(
                    width: 80,
                    child: TextField(
                      controller: quantityController,
                      decoration: InputDecoration(
                        labelText: 'Estoque',
                        border: OutlineInputBorder(),
                      ),
                      keyboardType: TextInputType.number,
                      inputFormatters: [
                        FilteringTextInputFormatter.digitsOnly,
                        LengthLimitingTextInputFormatter(5)
                      ],
                    ),
                  ),
                ],
              ),
              SizedBox(
                height: 20,
              ),
              ElevatedButton(
                  onPressed: () => _openFileExplorer(),
                  child: Text('Adicionar fotos')),
              SizedBox(
                height: 20,
              ),
              PicturesGrid(
                  pictures: pictures,
                  remotePicturesUrls:
                      _product != null ? _product.imagesUrls : [],
                  height: 130,
                  deleteButtonAction: _confirmRemovePicture),
              SizedBox(
                height: 20,
              ),
              Row(
                mainAxisAlignment: MainAxisAlignment.end,
                children: [
                  ElevatedButton(
                    onPressed: () {
                      Navigator.of(context).pop();
                    },
                    child: Text('Cancelar'),
                  ),
                  SizedBox(
                    width: 20,
                  ),
                  ElevatedButton(
                      onPressed: () {
                        _save(nameController.text, descriptionController.text,
                            price, weight, quantity, pictures);
                      },
                      child: Text('Salvar')),
                ],
              ),
            ],
          ),
        ),
      ),
      bottomNavigationBar: BotNavBar(
        role: UserType.PARTNER,
      ),
    );
  }
}
